/***************************************************************************
                       cdemu_kernel.h  -  description
                             -------------------
    begin                : 2002-12-29
    copyright            : 2003 by Robert Penz <robert.penz@outertech.com>
                           2003 by Mike Frysinger <vapier@netzero.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
 *                                                                         *
 ***************************************************************************/

#ifndef __CDEMU_KERNEL_H__
#define __CDEMU_KERNEL_H__

#include <linux/version.h>			/* lets support 2.4 and 2.6 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
#define	IS_LINUX_2_6
#else
#define	IS_LINUX_2_4
#endif

#define	CDEMU_MAJOR					62				/* LOCAL/EXPERIMENTAL US */
static const char *major_name = "cdemu";
#define	MAJOR_NR					CDEMU_MAJOR
#define	DEVICE_NR(device)			MINOR(device)	/* has no partition bits */
#define	DEVICE_NAME					"cdemu"			/* name for messaging */
#define	DEVICE_NO_RANDOM							/* no entropy to contribute */
#define	DEVICE_REQUEST				cdemu_request
#define	DEVICE_OFF(d)								/* do-nothing */

#include <linux/module.h>			/* try_module_get,module_put */
#include <linux/genhd.h>			/* struct gendisk */
#include <linux/blkdev.h>			/* block device operations */
#include <linux/cdrom.h>			/* cdrom declarations */
#include <linux/file.h>				/* fput */
#include <linux/highmem.h>			/* kmap functions */
#include <asm/current.h>			/* access to the 'current' struct */
#ifdef IS_LINUX_2_6
#include <linux/moduleparam.h>		/* MODULE_AUTHOR,module_parm */
#include <linux/buffer_head.h>		/* buffer operations */
#else
#include <linux/blk.h>				/* block device operations */
#include <asm/uaccess.h>			/* move memory to/from userspace */
#endif
#ifdef CONFIG_DEVFS_FS
#include <linux/devfs_fs_kernel.h>
#endif

#include "cdemu.h"
#define	CDEMU_MODULE_VERSION		CDEMU_VERSION

struct cdemu_device {
	int							number;
	int							refcnt;
	dev_t						device;
	tTrack						tracks[47];
	int							num_tracks;		/* number of tracks on the cd */
	struct file					*backing_file;
	int							loaded;
	int							uid;			/* user who mounted cd */
	int							media_changed;
	int							locked;
	int							pause;
	char						comment[CDEMU_COMMENT_SIZE];
	int							size;
	spinlock_t					lock;
	struct cdrom_device_info	info;
	struct gendisk				*gd;
	struct block_device			*bdev;
};

#ifdef CDEMU_DEBUG
#define	printk_debug(msg, args...) do { printk(KERN_DEBUG "cdemu:%i: " msg, __LINE__, ## args); } while(0)
#else
#define	printk_debug(msg, args...) do { ; } while(0)
#endif
#define printk_cdemu(lvl, msg, args...) do { printk(lvl "cdemu:%i: " msg, __LINE__, ## args); } while(0)

#define	AUDIO						0
#define	MODE1						1
#define	MODE2						2
#define	MODE1_2352					10
#define	MODE2_2352					20
#define	MODE1_2048					30
#define	MODE2_2336					40

#define	CDEMU_LOADCD				0x5400			/* load the virtual cd */
#define	CDEMU_UNLOADCD				0x5401			/* unload the virtual cd*/

static int							max_cdemus = 8;
static struct						cdemu_device *cdemu_dev;
#ifdef IS_LINUX_2_4
static int							*cdemu_sizes;
static int							*cdemu_blksizes;
#ifdef CONFIG_DEVFS_FS
static devfs_handle_t				devfs_handle;	/*  For the directory */
#endif
#endif

#ifdef IS_LINUX_2_6
#define	UP_MODULE_USE()		try_module_get(THIS_MODULE)
#define	DOWN_MODULE_USE()	module_put(THIS_MODULE)
#else
#define UP_MODULE_USE()		MOD_INC_USE_COUNT
#define DOWN_MODULE_USE()	MOD_DEC_USE_COUNT
#endif

#endif /* __CDEMU_KERNEL_H__ */
